@@ -30,7 +30,7 @@ from calculate.lib.cl_ini_parser import iniParser
 from calculate.lib.cl_print import color_print
 from calculate.lib.utils.ip import Pinger, IPError
 from calculate.lib.utils.files import (getModeFile, removeDir,
-                                       pathJoin, tarLinks, readFile,
+                                       pathJoin, tarLinks, readFile, writeFile,
                                        listDirectory, process, find, STDOUT,
                                        checkUtils)
 from calculate.lib.utils.samba import Samba, SambaError
@@ -44,6 +44,7 @@ from shutil import copy2
 from socket import gethostbyname
 import tarfile
 from calculate.desktop.desktop import Desktop
+from calculate.client.rsync import ProfileSyncer, ProfileSyncerError
 
 from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
 
@@ -1298,3 +1299,132 @@ class Client(commandServer, encrypt, Desktop):
                 return False
             return False
         return res
+
+    def get_syncer(self, remotehost, user, pwd):
+        return ProfileSyncer(remotehost, 2009, user, pwd)
+
+    def checkSync(self, remotehost):
+        status = self.get_syncer(remotehost, None, None).check()
+        self.clVars.Set('cl_client_rsync_profile_set',
+            "on" if status else "off",
+            force=True)
+        return status
+
+    def syncLoginProfileNew(self, host, username, pwd,
+                            uid, gid, homeDir,
+                            profileName, clearHomeDir=True):
+        """
+        Получить профиль пользователя из домена
+        """
+        ps = self.get_syncer(host, username, pwd)
+        # if currect server has any files then sync it
+        remoteProfile = "::profile/{}".format(profileName)
+        if not self.syncUserNew(ps, uid, gid, homeDir, "login",
+                                remoteProfile, host=host):
+            if re.search(r'\[sender\] change_dir "{resPath}" \(in {resName}\) '
+                'failed: No such file or directory'.format(
+                    resPath=profileName,
+                    resName="profile"), ps.output):
+                if clearHomeDir:
+                    # clean home directory
+                    if not self.clearHomeDir(homeDir):
+                        return False
+                return True
+            else:
+                self.printERROR(ps.output)
+                self.printERROR(_("Failed to execute rsync"))
+                return False
+        return True
+
+    def syncLogoutProfileNew(self, host, username, pwd,
+                             uid, gid, homeDir,
+                             profileName, skipList):
+        """
+        Закачать профиль пользователя в домен
+        """
+        skipList = [os.path.relpath(x, homeDir) for x in skipList]
+        ps = self.get_syncer(host, username, pwd)
+        # if currect server has any files then sync it
+        remoteProfile = "::profile/{}".format(profileName)
+        if listDirectory(homeDir):
+            with writeFile(os.path.join(homeDir,".logout")) as f:
+                f.write("SUCCESS")
+            if not self.syncUserNew(ps, uid, gid, homeDir, "logout",
+                                    remoteProfile, host=host, skipList=skipList):
+                self.printERROR(ps.output)
+                self.printERROR(_("Failed to execute rsync"))
+                return False
+        return True
+
+    def syncUserNew(self, ps, uid, gid, userHome, sync,
+                 remoteProfile,
+                 host="default", skipList=()):
+        """
+        Синхронизация профиля пользователя
+        """
+        execStr = ""
+        skipPaths = self.clVars.Get("cl_sync_skip_path")
+        if not skipPaths:
+            self.printERROR(
+                _("Variable 'cl_sync_skip_path' empty") % userHome)
+            return False
+        deletePaths = self.clVars.Get("cl_sync_del_path")
+        if not deletePaths:
+            deletePaths = []
+        excludePaths = ['--exclude=/%s'% x
+            for x in skipPaths + deletePaths + list(skipList)
+            if x not in {".logout"} ]
+        rsyncParams = []
+        if sync == "login":
+            if os.path.exists(userHome):
+                filterPath = ['--filter=P /%s'% x for x in skipPaths
+                    if x not in {".logout"}
+                ]
+                rsyncParams = ['--delete-excluded',
+                    '--delete'] + excludePaths + filterPath + [
+                        '-rlptgo', '-x'
+                    ]
+                source, target = remoteProfile, userHome
+        elif sync == "logout":
+            if os.path.exists(userHome) and os.listdir(userHome):
+                rsyncParams = ['--delete-excluded',
+                    '--delete'] + excludePaths + [
+                        '-rlpt', '-x'
+                    ]
+                source, target = userHome, remoteProfile
+        else:
+            raise ClientError(
+                _("Method syncUser: wrong option sync=%s") % str(sync))
+        if rsyncParams:
+            host = "<i>" + host + "</i>"
+            if sync == "login":
+                title = _("Fetching the user profile from %s") % host
+            elif sync == "logout":
+                title = _("Uploading the user profile to %s") % host
+            self.startTask(title)
+            self.addProgress()
+
+            for i in ps.sync("%s/"%source, "%s/"%target, *rsyncParams):
+                self.setProgress(i)
+
+            pathConfig = os.path.join(userHome,
+                                      self.pathConfig)
+            try:
+                if iniParser(configFileName).setVar(
+                        'rsync', {'exitcode': rsync.getExitCode()}):
+                    os.chmod(configFileName, 0600)
+                    os.chown(configFileName, uid, gid)
+            except Exception:
+                pass
+            if ps.exitstatus != 0:
+                return False
+        # change permissions
+        changeDirs = [userHome]
+        for changeDir in filter(path.exists, changeDirs):
+            # get directory permissions
+            mode = getModeFile(changeDir, mode="mode")
+            # if permission wrong( not 0700) then change it
+            if mode != 0700:
+                os.chmod(changeDir, 0700)
+        self.endTask()
+        return True
